;Car Alarm System (Transmitter) v1.0 1997
;Author: Kirill Yelizarov
;ver 1.1 2000 Alexandr Musatov
                    LIST          P=Pic12C509A, R=HEX
                    INCLUDE       p12c509a.inc
                    INCLUDE       alr97k01.pas  ;get password

                 __CONFIG _IntRC_OSC & _WDT_OFF & _CP_OFF & _MCLRE_OFF

;----- GPIO Port bits -----

IRLED             equ     2                   ;Infrared LED pin

;----- Local DATA SFRs -----


Count             equ    0x07
DelayCount        equ    0x08
DelayCountLow     equ    0x09
DelayCountHi      equ    0x0a


;----- Password SFRs -----

Dig1 	        equ   0x10
Dig2 		equ   0x11
Dig3 		equ   0x12
Dig4 		equ   0x13
Dig5 		equ   0x14
Dig6 		equ   0x15
Dig7 		equ   0x16
Dig8 		equ   0x17

;----- CODE -----

		org 0

		movwf		OSCCAL		;save oscillator calibration value

		clrf 		GPIO
		movlw 		b'00000000' 	;Enable weak pull-up on GP0 and GP1
		option 				;and wake up on pin change
		movlw		b'00111011'	;Set GP2 as output and the rest are inputs
		tris 		GPIO
		btfsc 		STATUS,GPWUF
		goto 		Transmit
		goto 		_Sleep

;Delay for timing intervals where actual delay is
;W*3-1+5 us with 4MHz oscillator

Delay
		movwf 		DelayCount
DelayStart:
		decfsz 		DelayCount,F
		goto 		DelayStart
		retlw 		0x00


;Delay for key de bounce where actual delay is
;W ms with 4MHz oscillator

LongDelay

		movwf 		DelayCountHi
		clrf 		DelayCountLow

DelayLoop:
		nop
		incfsz 		DelayCountLow,F
		goto 		DelayLoop
		decfsz 		DelayCountHi,F
		goto 		DelayLoop
		retlw 		0x00
Transmit:

		comf 		GPIO,W 		;read and invert GPIO
		andlw 		b'00001011'
		btfsc 		STATUS,Z 	;check if a button is pressed
		goto 		_Sleep 		;if not go to sleep
		movlw 		0x1e
		call 		LongDelay 	;wait for 30 ms
		comf 		GPIO,W 		;read and invert GPIO again
		andlw 		b'00001011'
		btfsc 		STATUS,Z 	;check if a button is still pressed
		goto 		_Sleep 		;if not go to sleep
		xorlw 		PASS1 		;xor pressed buttons with PASS1 digit
		movwf 		Dig1 		;assign password
		swapf 		Dig1,f
		movlw 		PASS2
		movwf 		Dig2
		movlw 		PASS3
		movwf 		Dig3
		movlw 		PASS4
		movwf 		Dig4
		movlw 		PASS5
		movwf 		Dig5
		movlw 		PASS6
		movwf 		Dig6
		movlw 		PASS7
		movwf 		Dig7
		movlw 		PASS8
		movwf 		Dig8
		movlw 		0x40
		movwf 		Count 		;64 bits transmission

		bsf 		GPIO,IRLED 	;Send START bit
		nop 				;each bit is a 14 us pulse
		nop 				;but the Start one is 20 us
		movlw 		0x04
		call 		Delay 		;delay 4*3-1+5=16 us
		bcf 		GPIO,IRLED
		movlw 		0xc3
		call 		Delay 		;delay195*3-1+5=589 us
		nop
		nop
		movlw 		0xc2
		call 		Delay 		;delay194*3-1+5=586 us
NextDig:
		bsf 		GPIO,IRLED 	;Send another bit
		bcf 		STATUS,C
		rrf 		Dig8,F
		rrf 		Dig7,F
		rrf 		Dig6,F
		rrf 		Dig5,F
		rrf 		Dig4,F
		rrf 		Dig3,F
		rrf 		Dig2,F
		rrf 		Dig1,F 		;rotate 64 bits to get the next bit
		nop 				;to transmit and place it into STATUS,C
		nop
		nop
		nop
		bcf 		GPIO,IRLED

		movlw 		0x0a
		call 		Delay 		;delay 10*3-1+5=34 us

		btfss 		STATUS,C 	;if STATUS,C is low
		goto 		Send0 		;then delay between bits is 400 us
		movlw 		0x84 		;else add another 400 us
		call 		Delay 		;delay 132*3-1+5=400 us
Send0:
		movlw 		0x71
		call 		Delay 		;delay 113*3-1+5=343 us

		decfsz 		Count,F 	;last bit is not reached
		goto 		NextDig


		nop

		bsf 		GPIO,IRLED 	;send END bit
		nop
		nop
		movlw 		0x02
		call 		Delay 		;delay 2*3-1+5=10 us
		bcf 		GPIO,IRLED


		movlw 		0xc4
		call 		Delay 		;delay196*3-1+5=592 us
		nop
		nop
		movlw 		0xc3
		call 		Delay 		;delay195*3-1+5=589 us

		bsf 		GPIO,IRLED 	;send STOP bit
		nop
		nop
		movlw 		0x02
		call 		Delay 		;delay 2*3-1+5=10 us
		bcf 		GPIO,IRLED
_Sleep:

		sleep 				;go to sleep

		end

