	; Car Alarm System (Main Unit) Version 01 K 1997
	; Author: Kirill Yelizarov
	;Version 02 k 2000 Alexandr Musatov
	LIST P=16f84, R=HEX
	INCLUDE "p16f84.inc"
	INCLUDE "alr97k01.pas"				;get password

	__CONFIG _XT_OSC & _WDT_OFF & _CP_OFF 

ALARM_TIME		equ        0x0a			;in intrusion mode make 10 signals

;----- Infrared input -----

	#define            IRinp	PORTA,2         ;infrared input is connected to GP3 pin

;----- Alarm Main Functions Timing Table -----
;one tick (tk) equals to 32 ms

;----- LED -----
	#define             LED         PORTA,1         ;LED is connected to GP4 pin
	#define             LED_d	Flags,4         ;LED direction flag
LED_f			equ         0x05		;LED flashing time (6 tk = 192 ms)
LED_p			equ         0x18		;LED pause between flashes time (25 tk = 800 ms)
	
;----- Lights -----
	#define             Lights      PORTA,0         ;Lights are connected to GP5 pin
	#define             Lights_d	Flags,5         ;Lights direction flag
Lights_f            equ         0x0f			;lights flashing time (15 tk = 480 ms)
Lights_p            equ         0x10			;lights pause between flashes (16 tk = 512 ms)

;----- Beep -----
	#define             Beep        PORTB,4         ;Horn is connected to GP0 pin
	#define             Beep_d	Flags,0         ;Beep direction flag
Beep_f              equ         0x08			;beep activate time (15 tk = 480 ms)
Beep_p              equ         0x10			;beep sleep time (16 tk = 512 ms)

;----- Lock doors -----
	#define		DLock		PORTB,7         ;Door lock relay and switch are connected to GP2 pin
	#define		DLock_d		Flags,2         ;Lock direction flag
	#define		open_f		Flags,6
	#define		close_f		Flags,7
DLock_f              equ         0x0b			;Lock activate time (22 tk = 723 ms)
DLock_p              equ         0x01			;should always be one

;----- Unlock doors -----
	#define             DUnlock	PORTB,6         ;Door unlock relay and switch are connected to GP1 pin
	#define             DUnlock_d   Flags,1         ;Unlock direction flag
DUnlock_f            equ          0x0b			;Unlock activate time (22 tk = 723 ms)
DUnlock_p            equ          0x01			;should always be one

;----- Input switch -----
	#define             InSw        PORTB,0		;Intrusion switch is connected to GP0
	#define             InSw_f      KeyFlags,0	;Intrusion flag
	#define          Intrusion      AlarmFlags,2

;----- Open switch -----
	#define           OpenSw         PORTB,2	;Door open switch is connected to GP1
	#define           OpenSw_f     KeyFlags,1	;Door open flag
	#define           Open        SwitchFlags,1

;----- Close switch -----
	#define          CloseSw         PORTB,3	;Door close switch is connected to GP2
	#define          CloseSw_f       KeyFlags,2	;Door close flag
	#define          Close         SwitchFlags,2
;------IMOBILIZER--------
	#define		Imob		PORTB,5
	#define		Kopot		PORTA,3
;----- AlarmFlags bits -----
NullFlag                equ     7
IRFlag                  equ     6
Disarm                  equ     5
Arm 			equ 	4
AlarmFlag 		equ 	3
TimerFlag 		equ 	1
BeepFlag 		equ 	0

;----- Local DATA -----
IRCount 		equ 	0x27 		;counter used in IRdelay
IRCorrection 		equ 	0x18 		;correction for infrared intervals
AlarmFlags 		equ 	0x19 		;alarm flags
Flags 			equ 	0x1a 		;direction flags
LED_c 			equ 	0x1b 		;LED counter
Lights_c 		equ 	0x0c 		;Lights counter
Beep_c 			equ 	0x0d 		;Beep counter
DLock_c 		equ 	0x0e 		;Lock counter
DUnlock_c 		equ 	0x0f 		;Unlock counter
Dig1 			equ 	0x10 		;64 bit code
Dig2 			equ 	0x11
Dig3 			equ 	0x12
Dig4 			equ 	0x13
Dig5 			equ 	0x14
Dig6 			equ 	0x15
Dig7 			equ 	0x16
Dig8 			equ 	0x17
KeyFlags 		equ 	0x28 		;key flags used by macro TestKey
AlarmCounter 		equ 	0x29 		;alarm counter
AlarmTris 		equ 	0x2a 		;alarm TRIS register
SwitchFlags 		equ 	0x2b 		;switch flags used by central lock
AlarmCounterT		equ	0x2c		;temp 
;----- Macro -----
Check 			macro 	r,rb,d,db,f,p,c
			local 	out,pas
;r - working bit
;d - direction flag
;f - flashing time (in ticks) (d flag is set)
;p - pause between flashes (in ticks) (d flag is cleared)
;c - counter
;delay 10us max
		decfsz  	c,F 			;decrease counter
		goto 		out
		btfss 		d,db 			;check direction
		goto		pas
		bcf 		r,rb 			;clear working bit
		bcf 		d,db 			;clear direction bit
		movlw 		p 			;read pause between flashes
		movwf 		c 			;and save it to counter
		goto		out 			;out from macro
pas: 
		bsf 		r,rb 			;set working bit
		bsf 		d,db 			;set direction bit
		movlw 		f 			;read flashing time
		movwf 		c 			;and save it to counter
out:
		endm

;This macro is used to search closed key
;button de bounce is 32 ms
;Delay 7us max

TestKey 	macro 	r,rb,f,fb,a,ab
 		local 		out,reset,setact
;r - pin to test
;f - pin flag (if pin r is low then set flag f, and check the next time pin r and flag f)
;a - actiun bit this pinb should be checked by the program and cleared by it

		btfsc 		r,rb 		;if pin is high then out and reset flag
		goto 		reset
		btfsc 		f,fb 		;skip if flag is cleared
		goto 		setact 		;else go and set action
		bsf 		f,fb 		;set flag and out
		goto 		out
setact:
		bsf 		a,ab 		;set action
reset:
		bcf 		f,fb 		;and clear flag
out:
		endm
;----- CODE -----

		org 0
		goto Start

; ------------ S U B R O U T I N E S -------------

Receive
		btfsc 		IRinp 		;wait till IRinp becomes low
		goto 		Receive

		clrf 		Dig1 		;~22 us
		clrf 		Dig2
		clrf 		Dig3
		clrf 		Dig4
		clrf 		Dig5
		clrf 		Dig6
		clrf 		Dig7
		clrf 		Dig8
		nop
		movlw 		0xb8
		call 		IRDelay 	;delay 184*3-1+5=556 us
		movlw 		0xb8
		call 		IRDelay 	;delay 184*3-1+5=556 us

;this is a special correction routine
;this is needed to sincronize transmitter and receiver,
;because they are clocked with internal RC generators

		clrf 		IRCorrection
		btfsc 		IRinp 		;0
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;1
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;2
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;3
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;4
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;5
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;6
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;7
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;8
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;9
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;10
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;11
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;12
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;13
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;14
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;15
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;16
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;17
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;18
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;19
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;20
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;21
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;22
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;23
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;24
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;25
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;26
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;27
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;28
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;29
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;30
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;31
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;32
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;33
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;34
		goto 		SetCorrection
		incf 		IRCorrection,F
		btfsc 		IRinp 		;35
		goto 		SetCorrection
		goto 		BadRead
SetCorrection:						;4 us
		movlw 		0x23			;23H=35. 
		movwf 		IRCount			;
SetCorrection0:
			 				;Delay 244+IRCorrection us
		movf 		IRCount,W 		;for T=400 us delay 261 us
		subwf 		IRCorrection,W
		btfsc 		STATUS,C
		goto 		AddCorrection0
AddCorrection0:
		decfsz 		IRCount,F
		goto 		SetCorrection0

		movlw 		0x26			;~4 us after start (6CH=108D)
		call 		IRDelay 		;108*3-1+5=328(delay 126*3-1+5=383 us)
		nop
		nop
		nop
	
		bsf 		STATUS,C 		;this STOP bit will end the loop
NextDigit:
		rrf 		Dig8,F
		rrf 		Dig7,F
		rrf 		Dig6,F
		rrf 		Dig5,F
		rrf 		Dig4,F
		rrf 		Dig3,F
		rrf 		Dig2,F
		rrf 		Dig1,F			;396 us
		btfsc 		STATUS,C 		;check for STOP bit
		goto 		Compare
		bsf 		AlarmFlags,NullFlag 	;Set the null flag
RetryDigit:
		bcf 		AlarmFlags,IRFlag 	;1 us Reset bit read flag  400us				
		nop					;402 us
		nop
		btfsc 		IRinp 			;5th us look for a bit    404us /804/1206
		bsf 		AlarmFlags,IRFlag 	;Set if bit read
MakeENDsinx:
		btfsc		IRinp
		goto		MakeENDsinx

		movlw 		0x23			;23H=35.
		movwf 		IRCount			;Delay 35*3-1=5+109 us

StartCorrection: 					;Delay 244+IRCorrection us
		movf 		IRCount,W 		;for T=400 us delay 261 us
		subwf 		IRCorrection,W
		btfsc 		STATUS,C
		goto 		AddCorrection
AddCorrection:
		decfsz 		IRCount,F
		goto 		StartCorrection
				
		movlw 		0x20			;23h--35dec
		call 		IRDelay 		;Delay 35*3-1+5=109 us

		btfss 		AlarmFlags,IRFlag
		goto 		ResetFlag
		btfsc 		AlarmFlags,NullFlag 	;If flag is clear it was 1
		goto 		Set0
		nop
		bsf 		STATUS,C 		;Set an overflow flag to read 1
		goto 		NextDigit
Set0:
		bcf 		STATUS,C 		;Reset an overflow flag to read 0
		goto 		NextDigit
ResetFlag:
		movlw 		0x05			;+_373
		call		IRDelay  		;Delay 2*3-1+5=10 us 5*3+4=19us
		nop					;19+1=20us
		btfss 		AlarmFlags,NullFlag	;Error if an overflow flag already reset
		goto 		BadRead
		bcf 		AlarmFlags,NullFlag
		goto 		RetryDigit
Compare:
		nop 					;401th us
		nop
		nop
		nop
		;btfsc 		IRinp 			;405th us watch for clear bit
		;goto 		BadRead
		movlw 		0x23
		movwf 		IRCount

StartCorrection1: 					;delay 244+IRCorrection us
		movf 		IRCount,W 		;for T=400 mks delay 261 us
		subwf 		IRCorrection,W
		btfsc 		STATUS,C
		goto 		AddCorrection1
AddCorrection1:
		decfsz 		IRCount,F
		goto 		StartCorrection1
		nop

;compare received data with password
		movlw 		PASS8
		xorwf 		Dig8,W
		btfss 		STATUS,Z
		goto 		BadCode

		movlw 		PASS7
		xorwf 		Dig7,W
		btfss 		STATUS,Z
		goto 		BadCode

		movlw 		PASS6
		xorwf 		Dig6,W
		btfss 		STATUS,Z
		goto 		BadCode

		movlw 		PASS5
		xorwf 		Dig5,W
		btfss 		STATUS,Z
		goto 		BadCode

		movlw 		PASS4
		xorwf 		Dig4,W
		btfss 		STATUS,Z
		goto 		BadCode

		movlw 		PASS3
		xorwf 		Dig3,W
		btfss 		STATUS,Z
		goto 		BadCode

		movlw 		PASS2
		xorwf 		Dig2,W
		btfss 		STATUS,Z
		goto 		BadCode

		movf 		Dig1,W
		andlw 		b'00001111'
		xorlw 		PASS1
		btfss 		STATUS,Z
		goto 		BadCode

		movlw 		0x20 				;
		call 		IRDelay 			;delay 32*3-1+5=100 us
		nop
		;btfsc 		IRinp 				;805 us
		;goto 		BadRead

		movlw 		0x23
		movwf 		IRCount

StartCorrection2: 						;elay 244+IRCorrection us

		movf 		IRCount,W 			;for T=400 us delay 261 us
		subwf 		IRCorrection,W
		btfsc 		STATUS,C
		goto 		AddCorrection2
AddCorrection2:
		decfsz 		IRCount,F
		goto 		StartCorrection2
		nop

		movlw 		0x2b
		call 		IRDelay 			;delay 43*3-1+5=133 us
		nop

		;btfss 		IRinp 				;5th us look for a STOP bit
		;goto 		BadRead
		

		movlw 		b'00110000'
		andwf 		Dig1,W
		btfsc 		STATUS,Z
		goto 		BadRead
		iorwf 		AlarmFlags,F 			;Save function (ON/OFF)	

		retlw 		0x00
		
BadCode:
		bsf 		Intrusion 		;Set an intrusion flag for bad code
	
BadRead:
		retlw		0x00


;Delay for timing intervals where actual delay is
;W*3-1+5 us with 4MHz oscillator

IRDelay
		movwf 		IRCount
DelayStart:
		decfsz 		IRCount,F
		goto 		DelayStart
		retlw 		0x00

;	 ------------ M A I N -------------
Start:
		clrf		PORTB
		clrf 		PORTA
		bsf		STATUS,RP0
		movlw		b'01000111' 		;Enable weak pull-ups and wake up on pin change
		movwf		OPTION_REG		;and set prescaller to 1:256		
		movlw 		b'00001111' 		;Set GP4 an GP5 as output and the rest are inputs
		movwf		TRISB
		movlw		b'00000100'
		movwf		TRISA			;save data to alarm tris register
		bcf		STATUS,RP0
		clrf 		AlarmFlags 		;clear alarm flags
		clrf 		Flags 			;clear direction flags
		clrf 		SwitchFlags 		;clear central lock switch flags
		movlw 		0x01
		movwf 		Beep_c
		movwf 		Lights_c
		movwf 		LED_c
		movwf 		DLock_c
		movwf 		DUnlock_c	
		;goto 		MainLoop
MainLoop:
		btfsc 		IRinp 			;Check IR input pin
		call 		Receive
		btfss 		TMR0,7			;if timer0 highest bit is cleared 
		goto 		MainLoop						

	
;MainLoop:
	
Tick:		
		clrf		TMR0
		;bcf 		AlarmFlags,TimerFlag 	;clear TimerFlag
		btfsc 		IRinp 			;Check IR input pin
		call 		Receive
		btfsc 		Open
		goto 		_Open
		btfsc 		Close
		goto 		_Close
		btfss 		AlarmFlags,AlarmFlag
		goto 		_Armed 			;if falg is cleared it is arm mode
		goto 		_Disarmed
_Armed:
		btfsc 		Intrusion
		goto 		_Intrusion
		btfsc 		IRinp 				;Check IR input pin 11us
		call 		Receive
		bsf		Beep
		bcf		Lights
		Check 		LED,LED_d,LED_f,LED_p,LED_c	;check LED (delay 10us)
		bcf		Imob				;IMMOBILAZATION
		btfsc 		IRinp 				;Check IR input pin
		call 		Receive
		TestKey 	InSw,InSw_f,Intrusion 		;test intrusion switch (delay 7us)
		movlw 		ALARM_TIME
		movwf 		AlarmCounter 			;set alarm counter
		movlw		0x1f
		movwf		AlarmCounterT
		btfsc 		AlarmFlags,Disarm
		goto 		DisarmAlarm
		goto 		MainLoop
_Intrusion:
		btfsc 		AlarmFlags,Disarm
		goto 		DisarmAlarm
	
		bsf 		LED 					;turn on LED
		btfsc 		IRinp 					;Check IR input pin
		call 		Receive
		bcf		Beep
		Check 		Lights,Lights_d,Lights_f,Lights_p,Lights_c 	;check Lights (delay 10us)		
		decfsz 		AlarmCounterT,F
		goto 		MainLoop
		movlw		0x1f
		movwf		AlarmCounterT
		decfsz 		AlarmCounter,F
		goto 		MainLoop
		bsf		Beep
		bcf 		Intrusion
		bcf 		LED 						;turn off LED
		goto 		MainLoop
_Disarmed:
		bsf		Beep
		bcf 		LED 					;turn LED off
		bsf		Imob	
		btfsc 		IRinp 					;Check IR input pin
		call 		Receive
		TestKey 	OpenSw,OpenSw_f,Open 			;test open switch (delay 7us)
		bcf		Lights
		btfsc 		IRinp 					;Check IR input pin
		call 		Receive
		TestKey 	CloseSw,CloseSw_f,Close 		;test close switch (delay 7us)
		btfsc 		AlarmFlags,Arm
		goto 		ArmAlarm
		goto 		MainLoop
_Open:

		btfsc		open_f
		goto		Ma1
		
		btfsc 		IRinp 					;Check IR input pin
		call 		Receive
		
		Check 	DUnlock,DUnlock_d,DUnlock_f,DUnlock_p,DUnlock_c			;check Unlock (delay 10us)
		btfsc 		DUnlock_d 				;wait till direction flag changes its state
		goto 		MainLoop
Ma1:		
		bsf		open_f
		bcf		close_f
		bcf 		Open
		goto 		MainLoop
_Close:

		btfsc		close_f
		goto		Ma2	
		btfsc 		IRinp 					;Check IR input pin
		call 		Receive
		Check 		DLock,DLock_d,DLock_f,DLock_p,DLock_c	;check Lock (delay 10us)
		btfsc 		DLock_d	 				;wait till direction flag changes its state
		goto 		MainLoop
Ma2:		
		bsf		close_f
		bcf		open_f
		bcf 		Close	
		goto 		MainLoop

ArmAlarm:	 				
		bcf		Beep
		bsf		Lights
		movlw		Beep_f
		movwf		Beep_c
TikB:		
		clrf		TMR0
	
Beeep:										
		btfss 		TMR0,7				;if timer0 highest bit is cleared 
		goto 		Beeep					
		decfsz  	Beep_c,f 			;decrease counter
		goto 		TikB		
		bsf		Beep
		bcf		Lights

		bcf 		AlarmFlags,Arm
		bcf 		AlarmFlags,Disarm
		bcf		close_f	
		bsf 		Close
		bcf		AlarmFlags,AlarmFlag
		goto 		MainLoop

DisarmAlarm:

		bcf		Beep
		bsf		Lights
		movlw		Beep_f
		movwf		Beep_c
DTikB:		
		clrf		TMR0
	
DBeeep:										
		btfss 		TMR0,7				;if timer0 highest bit is cleared 
		goto 		DBeeep					
		decfsz  	Beep_c,f 			;decrease counter
		goto 		DTikB		
		bsf		Beep
		bcf		Lights

		bcf		open_f
		bcf 		AlarmFlags,Disarm
		bcf 		AlarmFlags,Arm
		bsf 		Open
		bsf		AlarmFlags,AlarmFlag
		bcf 		Intrusion

		goto 		MainLoop

	

		end



		

